==> rulee-ts-0.13.0.tgz: package/lib/esm/engine/builder/builder.js <==
import { PropertyScalarImpl } from "../../properties/property-scalar-impl";
import { PropertyScalarRuleBuilder } from "./property-scalar-rule-builder";
import { PropertyScalarBuilder } from "./property-scalar-builder";
import { TriggerBuilder } from "./trigger-builder";
import { GroupOfPropertiesBuilder } from "./group-of-properties-builder";
import { GroupOfPropertiesImpl } from "../../properties/group-of-properties-impl";
import { V } from "../../validators/common/common-validators";
import { ListBuilder, SelectionMode } from "./list-builder";
import { ListOfPropertiesImpl } from "../../properties/list-of-properties-impl";
import { PropertyArrayListAsyncImpl, PropertyArrayListSyncImpl } from "../../properties/property-array-list-impl";
import { TextInterpreter } from "../../util/text-interpreter/text-interpreter";
import { GroupOfPropertiesRuleBuilder } from "./group-of-properties-rule-builder";
import { ListOfPropertiesRuleBuilder } from "./list-of-properties-rule-builder";
import { PropertyArrayListRuleBuilder } from "./property-array-list-rule-builder";
export class Builder {
    constructor(options, ruleEngine, dependencyGraph, propertyMap) {
        var _a;
        this.ruleEngine = ruleEngine;
        this.dependencyGraph = dependencyGraph;
        this.propertyMap = propertyMap;
        this.trigger = new TriggerBuilder();
        this.notEmptyIfRequiredValidator = options.emptyButRequiredMessage instanceof Function
            ? V.scalar.notEmptyMsgProvider(options.emptyButRequiredMessage)
            : V.scalar.notEmpty(options.emptyButRequiredMessage);
        if (options.defaultEmptyChoiceDisplayValue) {
            this.defaultEmptyChoiceDisplayValue = options.defaultEmptyChoiceDisplayValue;
        }
        this.defaultBackpressureConfig = Object.freeze((_a = options.defaultBackpressureConfig) !== null && _a !== void 0 ? _a : {
            type: "switch",
            debounceTime: 40
        });
        this.textInterpreters = {
            [TextInterpreter.Html]: options.textInterpreterHtml,
            [TextInterpreter.Markdown]: options.textInterpreterMarkdown,
            [TextInterpreter.Custom]: options.textInterpreterCustom,
        };
        this.list = new ListBuilder((id, itemTemplate, config) => this.listOfProperties(id, itemTemplate, config), (id, provider, dependencies) => this.propertyList(id, provider, dependencies), (id, provider, dependencies, propertyConfig) => this.asyncPropertyList(id, provider, dependencies, propertyConfig), (prop) => this.bindListOfProperties(prop), (prop) => this.bindPropertyArrayList(prop));
        this.group = new GroupOfPropertiesBuilder((id, properties) => this.groupOfProperties(id, properties), (prop) => this.bindGroupOfProperties(prop));
        this.scalar = new PropertyScalarBuilder((id, provider, emptyValueFcn, converter, dependencies, propertyConfig, ownedProperties) => this.propertyScalar(id, provider, emptyValueFcn, converter, dependencies, propertyConfig, ownedProperties), (prop) => this.bindPropertyScalar(prop), this.defaultEmptyChoiceDisplayValue, this.list);
    }
    get properties() {
        return Object.values(this.propertyMap);
    }
    propertyScalar(id, provider, emptyValueFcn, converter, dependencies, config, ownedProperties) {
        var _a;
        const prop = new PropertyScalarImpl(id, provider, emptyValueFcn, converter, this.ruleEngine, (_a = config === null || config === void 0 ? void 0 : config.backpressure) !== null && _a !== void 0 ? _a : (provider.isAsynchronous() ? this.defaultBackpressureConfig : undefined));
        this.addProperty(prop);
        if (dependencies) {
            this.addDependencies(this.dependencyGraph, dependencies, prop, { value: true });
        }
        if (ownedProperties) {
            ownedProperties.forEach(owned => this.dependencyGraph.addOwnerDependency(prop, owned, false));
        }
        return prop;
    }
    bindPropertyScalar(prop) {
        return new PropertyScalarRuleBuilder(prop, this.notEmptyIfRequiredValidator, (from, to, options) => this.addDependencies(this.dependencyGraph, from, to, options), this.textInterpreters);
    }
    groupOfProperties(id, properties) {
        const prop = new GroupOfPropertiesImpl(id, properties, this.ruleEngine);
        this.addProperty(prop);
        this.addDependencies(this.dependencyGraph, prop.propertiesAsList, prop, { value: true });
        return prop;
    }
    bindGroupOfProperties(prop) {
        return new GroupOfPropertiesRuleBuilder(prop, (from, to, options) => this.addDependencies(this.dependencyGraph, from, to, options));
    }
    listOfProperties(id, itemTemplate, config) {
        const isMultiSelect = (config === null || config === void 0 ? void 0 : config.selectionMode) === SelectionMode.MultiSelect;
        const prop = new ListOfPropertiesImpl(id, itemTemplate, isMultiSelect, this.ruleEngine, this.dependencyGraph);
        this.addProperty(prop);
        return prop;
    }
    bindListOfProperties(prop) {
        return new ListOfPropertiesRuleBuilder(prop, (from, to, options) => this.addDependencies(this.dependencyGraph, from, to, options));
    }
    propertyList(id, provider, dependencies) {
        const prop = new PropertyArrayListSyncImpl(id, provider, this.ruleEngine, undefined);
        this.addProperty(prop);
        if (dependencies) {
            this.addDependencies(this.dependencyGraph, dependencies, prop, { value: true });
        }
        return prop;
    }
    asyncPropertyList(id, provider, dependencies, config) {
        const prop = new PropertyArrayListAsyncImpl(id, provider, this.ruleEngine, config === null || config === void 0 ? void 0 : config.backpressure);
        this.addProperty(prop);
        if (dependencies) {
            this.addDependencies(this.dependencyGraph, dependencies, prop, { value: true });
        }
        return prop;
    }
    bindPropertyArrayList(prop) {
        return new PropertyArrayListRuleBuilder(prop, (from, to, options) => this.addDependencies(this.dependencyGraph, from, to, options));
    }
    addDependencies(dependencyGraph, from, to, options) {
        if (from.length) {
            dependencyGraph.addDependencies(from, to, options);
        }
    }
    addProperty(property) {
        if (this.propertyMap[property.id]) {
            throw new Error(`A Property with the id ${property.id} is already defined`);
        }
        this.propertyMap[property.id] = property;
    }
    defineCustomAttribute(name) {
        return { name };
    }
    addCrossValidator(...validatedProperties) {
        return (validator) => {
            const instance = {
                validationArguments: validatedProperties,
                validate: validator
            };
            validatedProperties.forEach((prop, i) => {
                prop.addValidator(instance);
                for (let j = i + 1; j < validatedProperties.length; j++) {
                    const dependent = validatedProperties[j];
                    this.dependencyGraph.addDependency(prop, dependent, { validation: true });
                    this.dependencyGraph.addDependency(dependent, prop, { validation: true });
                }
            });
        };
    }
    decorateTemplate(template, decorator) {
        return (id, index, siblingAccess) => {
            const property = template(id, index, siblingAccess);
            decorator(property);
            return property;
        };
    }
    asVisJsData() {
        return this.dependencyGraph.createVisJsData(this.properties);
    }
    generateNetworkGraphHtmlPage() {
        const graph = this.asVisJsData();
        const nodes = JSON.stringify(graph.nodes);
        const edges = JSON.stringify(graph.edges);
        const options = JSON.stringify(graph.options);
        return `<!DOCTYPE HTML>
<html>
<head>
    <script src="https://visjs.github.io/vis-network/standalone/umd/vis-network.min.js"></script>
    <style>
        #network {
            position: absolute; top: 0; right: 0; bottom: 0; left: 0;
        }
    </style>
</head>
<body>
    <div id="network"></div>
    <script type="text/javascript">
        var container = document.getElementById("network");
        var data = {
            nodes: new vis.DataSet(${nodes}),
            edges: new vis.DataSet(${edges}),
        };
        var options = ${options};
        var network = new vis.Network(container, data, options);
    </script> 
</body>
</html>`;
    }
}

==> rulee-ts-0.14.0.tgz: package/lib/esm/engine/builder/builder.js <==
import { PropertyScalarImpl } from "../../properties/property-scalar-impl";
import { PropertyScalarRuleBuilder } from "./property-scalar-rule-builder";
import { PropertyScalarBuilder } from "./property-scalar-builder";
import { TriggerBuilder } from "./trigger-builder";
import { GroupOfPropertiesBuilder } from "./group-of-properties-builder";
import { GroupOfPropertiesImpl } from "../../properties/group-of-properties-impl";
import { V } from "../../validators/common/common-validators";
import { ListBuilder, SelectionMode } from "./list-builder";
import { ListOfPropertiesImpl } from "../../properties/list-of-properties-impl";
import { PropertyArrayListAsyncImpl, PropertyArrayListSyncImpl } from "../../properties/property-array-list-impl";
import { TextInterpreter } from "../../util/text-interpreter/text-interpreter";
import { GroupOfPropertiesRuleBuilder } from "./group-of-properties-rule-builder";
import { ListOfPropertiesRuleBuilder } from "./list-of-properties-rule-builder";
import { PropertyArrayListRuleBuilder } from "./property-array-list-rule-builder";
export class Builder {
    constructor(options, ruleEngine, dependencyGraph, propertyMap) {
        var _a;
        this.ruleEngine = ruleEngine;
        this.dependencyGraph = dependencyGraph;
        this.propertyMap = propertyMap;
        this.trigger = new TriggerBuilder();
        this.notEmptyIfRequiredValidator = options.emptyButRequiredMessage instanceof Function
            ? V.scalar.notEmptyMsgProvider(options.emptyButRequiredMessage)
            : V.scalar.notEmpty(options.emptyButRequiredMessage);
        if (options.defaultEmptyChoiceDisplayValue) {
            this.defaultEmptyChoiceDisplayValue = options.defaultEmptyChoiceDisplayValue;
        }
        this.defaultBackpressureConfig = Object.freeze((_a = options.defaultBackpressureConfig) !== null && _a !== void 0 ? _a : {
            type: "switch",
            debounceTime: 40
        });
        this.textInterpreters = {
            [TextInterpreter.Html]: options.textInterpreterHtml,
            [TextInterpreter.Markdown]: options.textInterpreterMarkdown,
            [TextInterpreter.Custom]: options.textInterpreterCustom,
        };
        this.list = new ListBuilder((id, itemTemplate, config) => this.listOfProperties(id, itemTemplate, config), (id, provider, dependencies) => this.propertyList(id, provider, dependencies), (id, provider, dependencies, propertyConfig) => this.asyncPropertyList(id, provider, dependencies, propertyConfig), (prop) => this.bindListOfProperties(prop), (prop) => this.bindPropertyArrayList(prop));
        this.group = new GroupOfPropertiesBuilder((id, properties) => this.groupOfProperties(id, properties), (prop) => this.bindGroupOfProperties(prop));
        this.scalar = new PropertyScalarBuilder((id, provider, emptyValueFcn, converter, dependencies, propertyConfig, ownedProperties) => this.propertyScalar(id, provider, emptyValueFcn, converter, dependencies, propertyConfig, ownedProperties), (prop) => this.bindPropertyScalar(prop), this.defaultEmptyChoiceDisplayValue, this.list);
    }
    get properties() {
        return Object.values(this.propertyMap);
    }
    propertyScalar(id, provider, emptyValueFcn, converter, dependencies, config, ownedProperties) {
        var _a;
        const prop = new PropertyScalarImpl(id, provider, emptyValueFcn, converter, this.ruleEngine, (_a = config === null || config === void 0 ? void 0 : config.backpressure) !== null && _a !== void 0 ? _a : (provider.isAsynchronous() ? this.defaultBackpressureConfig : undefined));
        this.addProperty(prop);
        if (dependencies) {
            this.addDependencies(this.dependencyGraph, dependencies, prop, { value: true });
        }
        if (ownedProperties) {
            ownedProperties.forEach(owned => this.dependencyGraph.addOwnerDependency(prop, owned, false));
        }
        return prop;
    }
    bindPropertyScalar(prop) {
        return new PropertyScalarRuleBuilder(prop, this.notEmptyIfRequiredValidator, (from, to, options) => this.addDependencies(this.dependencyGraph, from, to, options), this.textInterpreters);
    }
    groupOfProperties(id, properties) {
        const prop = new GroupOfPropertiesImpl(id, properties, this.ruleEngine);
        this.addProperty(prop);
        this.addDependencies(this.dependencyGraph, prop.propertiesAsList, prop, { value: true });
        prop.propertiesAsList.forEach(owned => this.dependencyGraph.addOwnerDependency(prop, owned, false));
        return prop;
    }
    bindGroupOfProperties(prop) {
        return new GroupOfPropertiesRuleBuilder(prop, (from, to, options) => this.addDependencies(this.dependencyGraph, from, to, options), this.textInterpreters);
    }
    listOfProperties(id, itemTemplate, config) {
        const isMultiSelect = (config === null || config === void 0 ? void 0 : config.selectionMode) === SelectionMode.MultiSelect;
        const prop = new ListOfPropertiesImpl(id, itemTemplate, isMultiSelect, this.ruleEngine, this.dependencyGraph);
        this.addProperty(prop);
        return prop;
    }
    bindListOfProperties(prop) {
        return new ListOfPropertiesRuleBuilder(prop, (from, to, options) => this.addDependencies(this.dependencyGraph, from, to, options), this.textInterpreters);
    }
    propertyList(id, provider, dependencies) {
        const prop = new PropertyArrayListSyncImpl(id, provider, this.ruleEngine, undefined);
        this.addProperty(prop);
        if (dependencies) {
            this.addDependencies(this.dependencyGraph, dependencies, prop, { value: true });
        }
        return prop;
    }
    asyncPropertyList(id, provider, dependencies, config) {
        const prop = new PropertyArrayListAsyncImpl(id, provider, this.ruleEngine, config === null || config === void 0 ? void 0 : config.backpressure);
        this.addProperty(prop);
        if (dependencies) {
            this.addDependencies(this.dependencyGraph, dependencies, prop, { value: true });
        }
        return prop;
    }
    bindPropertyArrayList(prop) {
        return new PropertyArrayListRuleBuilder(prop, (from, to, options) => this.addDependencies(this.dependencyGraph, from, to, options), this.textInterpreters);
    }
    addDependencies(dependencyGraph, from, to, options) {
        if (from.length) {
            dependencyGraph.addDependencies(from, to, options);
        }
    }
    addProperty(property) {
        if (this.propertyMap[property.id]) {
            throw new Error(`A Property with the id ${property.id} is already defined`);
        }
        this.propertyMap[property.id] = property;
    }
    defineCustomAttribute(name) {
        return { name };
    }
    addCrossValidator(...validatedProperties) {
        return (validator) => {
            const instance = {
                validationArguments: validatedProperties,
                validate: validator
            };
            validatedProperties.forEach((prop, i) => {
                prop.addValidator(instance);
                for (let j = i + 1; j < validatedProperties.length; j++) {
                    const dependent = validatedProperties[j];
                    this.dependencyGraph.addDependency(prop, dependent, { validation: true });
                    this.dependencyGraph.addDependency(dependent, prop, { validation: true });
                }
            });
        };
    }
    decorateTemplate(template, decorator) {
        return (id, index, siblingAccess) => {
            const property = template(id, index, siblingAccess);
            decorator(property);
            return property;
        };
    }
    asVisJsData() {
        return this.dependencyGraph.createVisJsData(this.properties);
    }
    generateDependencyGraphHtmlPage() {
        const graph = this.asVisJsData();
        const nodes = JSON.stringify(graph.nodes);
        const edges = JSON.stringify(graph.edges);
        const options = JSON.stringify(graph.options);
        return `<!DOCTYPE HTML>
<html>
<head>
    <script src="https://visjs.github.io/vis-network/standalone/umd/vis-network.min.js"></script>
    <style>
        #network {
            position: absolute; top: 0; right: 0; bottom: 0; left: 0;
        }
    </style>
</head>
<body>
    <div id="network"></div>
    <script type="text/javascript">
        var container = document.getElementById("network");
        var data = {
            nodes: new vis.DataSet(${nodes}),
            edges: new vis.DataSet(${edges}),
        };
        var options = ${options};
        var network = new vis.Network(container, data, options);
    </script> 
</body>
</html>`;
    }
}
